
/* Chrysalide - Outil d'analyse de fichiers binaires
 * literal.c - équivalent Python du fichier "analysis/scan/exprs/literal.c"
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "literal.h"


#include <pygobject.h>


#include <i18n.h>
#include <analysis/scan/exprs/literal-int.h>


#include "constants.h"
#include "../expr.h"
#include "../../../access.h"
#include "../../../helpers.h"



CREATE_DYN_CONSTRUCTOR(scan_literal_expression, G_TYPE_SCAN_LITERAL_EXPRESSION);

/* Initialise une instance sur la base du dérivé de GObject. */
static int py_scan_literal_expression_init(PyObject *, PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = objet à initialiser (théoriquement).                  *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Initialise une instance sur la base du dérivé de GObject.    *
*                                                                             *
*  Retour      : 0.                                                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int py_scan_literal_expression_init(PyObject *self, PyObject *args, PyObject *kwds)
{
    PyObject *py_value;                     /* Valeur en version Python    */
    int ret;                                /* Bilan de lecture des args.  */
    LiteralValueType vtype;                 /* Valeur à porter             */
    bool arg_boolean;                       /* Argument natif booléen      */
    unsigned long long arg_uinteger;        /* Argument natif entier       */
    sized_string_t arg_string;              /* Argument natif textuel      */
    Py_ssize_t arg_str_length;              /* Taille de ce texte          */
    void *arg_ptr;                          /* Pointeur vers un argument   */
    GScanLiteralExpression *expr;           /* Création GLib à transmettre */

#define SCAN_LITERAL_EXPRESSION_DOC                                 \
    "A ScanLiteralExpression object defines expression carrying"    \
    " literal values available for scan match conditions.\n"        \
    "\n"                                                            \
    "Instances can be created using one of the following"           \
    " constructors:\n"                                              \
    "\n"                                                            \
    "    ScanLiteralExpression(value)"                              \
    "\n"                                                            \
    "\n"                                                            \
    "Where *value* is either a boolean, an integer or bytes."

    /* Récupération des paramètres */

    ret = PyArg_ParseTuple(args, "O", &py_value);
    if (!ret) return -1;

    if (PyBool_Check(py_value))
    {
        vtype = LVT_BOOLEAN;

        arg_boolean = (py_value == Py_True);
        arg_ptr = &arg_boolean;

    }

    else if (PyLong_Check(py_value))
    {
        if (1 /* sign - TODO */)
            ;

        vtype = LVT_UNSIGNED_INTEGER;

        arg_uinteger = PyLong_AsUnsignedLongLong(py_value);
        arg_ptr = &arg_uinteger;

    }

    else if (PyBytes_Check(py_value))
    {
        vtype = LVT_STRING;

        ret = PyBytes_AsStringAndSize(py_value, &arg_string.data, &arg_str_length);
        if (ret == -1) return -1;

        arg_string.len = arg_str_length;
        arg_ptr = &arg_string;

    }

    else
    {
        PyErr_SetString(PyExc_ValueError, _("Unsupported Python value for a literal scan expression."));
        return -1;
    }

    /* Initialisation d'un objet GLib */

    ret = forward_pygobjet_init(self);
    if (ret == -1) return -1;

    /* Eléments de base */

    expr = G_SCAN_LITERAL_EXPRESSION(pygobject_get(self));

    if (!g_scan_literal_expression_create(expr, vtype, arg_ptr))
    {
        PyErr_SetString(PyExc_ValueError, _("Unable to create literal expression."));
        return -1;
    }

    return 0;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_scan_literal_expression_type(void)
{
    static PyMethodDef py_scan_literal_expression_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_scan_literal_expression_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_scan_literal_expression_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.scan.exprs.ScanLiteralExpression",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = SCAN_LITERAL_EXPRESSION_DOC,

        .tp_methods     = py_scan_literal_expression_methods,
        .tp_getset      = py_scan_literal_expression_getseters,

        .tp_init        = py_scan_literal_expression_init,
        .tp_new         = py_scan_literal_expression_new,

    };

    return &py_scan_literal_expression_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide....PlainModifier'.     *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_scan_literal_expression_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'PlainModifier' */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_scan_literal_expression_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.analysis.scan.exprs");

        dict = PyModule_GetDict(module);

        if (!ensure_python_scan_expression_is_registered())
            return false;

        if (!register_class_for_pygobject(dict, G_TYPE_SCAN_LITERAL_EXPRESSION, type))
            return false;

        if (!define_literal_expression_value_type_constants(type))
            return false;

    }

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : arg = argument quelconque à tenter de convertir.             *
*                dst = destination des valeurs récupérées en cas de succès.   *
*                                                                             *
*  Description : Tente de convertir en transmission d'octets à l'identique.   *
*                                                                             *
*  Retour      : Bilan de l'opération, voire indications supplémentaires.     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

int convert_to_scan_literal_expression(PyObject *arg, void *dst)
{
    int result;                             /* Bilan à retourner           */

    result = PyObject_IsInstance(arg, (PyObject *)get_python_scan_literal_expression_type());

    switch (result)
    {
        case -1:
            /* L'exception est déjà fixée par Python */
            result = 0;
            break;

        case 0:
            PyErr_SetString(PyExc_TypeError, "unable to convert the provided argument to scan literal expression");
            break;

        case 1:
            *((GScanLiteralExpression **)dst) = G_SCAN_LITERAL_EXPRESSION(pygobject_get(arg));
            break;

        default:
            assert(false);
            break;

    }

    return result;

}
